@@ -11,14 +11,13 @@
 #include "sysdeps.h"
 #include <assert.h>
 
-#include "config.h"
 #include "options.h"
 #include "events.h"
 #include "memory.h"
 #include "custom.h"
 #include "cia.h"
 
-#include "uade.h"
+#include "uadectl.h"
 
 #define DIV10 5 /* Yes, a bad identifier. */
 
@@ -251,7 +250,7 @@ void CIA_hsync_handler(void)
 
 }
 
-void CIA_vsync_handler()
+void CIA_vsync_handler(void)
 {
     if (ciaatodon)
 	ciaatod++;
@@ -373,16 +372,11 @@ static void WriteCIAA(uae_u16 addr,uae_u8 val)
 	oldovl = ciaapra & 1;
 	oldled = ciaapra & 2;
 	ciaapra = (ciaapra & ~0x3) | (val & 0x3); 
-	gui_ledstate &= ~1;
+	gui_ledstate = 0;
 	if (!gui_ledstate_forced) {
-	  gui_ledstate |= ((~ciaapra & 2) >> 1);
+	  gui_ledstate = (~ciaapra & 2) >> 1;
 	} else {
-	  gui_ledstate |= gui_ledstate_forced & 1 ? 1 : 0;
-	}
-	if (uade_filter_debug) {
-	  if (((int) (gui_ledstate & 1)) == (oldled >> 1)) {
-	    fprintf(stderr, "uade: led filter %s\n", (gui_ledstate & 1) ? "on" : "off");
-	  }
+	  gui_ledstate = gui_ledstate_forced & 1;
 	}
 	/* we don't want to have ersatzkickfile or kickstart roms in uade */
 	/*
@@ -626,6 +620,7 @@ void CIA_reset(void)
     lastdiv10 = 0;
     CIA_calctimers();
     ciabpra = 0x8C;
+
     /* we don't want ersatzkick or kickstart roms in uade */
     /*
       if (! ersatzkickfile)
@@ -658,17 +653,17 @@ addrbank cia_bank = {
     default_xlate, default_check
 };
 
-uae_u32 REGPARAM2 cia_lget (uaecptr addr)
+static uae_u32 REGPARAM2 cia_lget (uaecptr addr)
 {
     return cia_bget(addr+3);
 }
 
-uae_u32 REGPARAM2 cia_wget (uaecptr addr)
+static uae_u32 REGPARAM2 cia_wget (uaecptr addr)
 {
     return cia_bget(addr+1);
 }
 
-uae_u32 REGPARAM2 cia_bget (uaecptr addr)
+static uae_u32 REGPARAM2 cia_bget (uaecptr addr)
 {
     if ((addr & 0x3001) == 0x2001)
 	return ReadCIAA((addr & 0xF00) >> 8);
@@ -677,17 +672,17 @@ uae_u32 REGPARAM2 cia_bget (uaecptr addr)
     return 0;
 }
 
-void REGPARAM2 cia_lput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 cia_lput (uaecptr addr, uae_u32 value)
 {
     cia_bput(addr+3,value); /* FIXME ? */
 }
 
-void REGPARAM2 cia_wput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 cia_wput (uaecptr addr, uae_u32 value)
 {
     cia_bput(addr+1,value);
 }
 
-void REGPARAM2 cia_bput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 cia_bput (uaecptr addr, uae_u32 value)
 {
     if ((addr & 0x3001) == 0x2001)
 	WriteCIAA((addr & 0xF00) >> 8,value);
@@ -710,19 +705,19 @@ addrbank clock_bank = {
     default_xlate, default_check
 };
 
-uae_u32 REGPARAM2 clock_lget (uaecptr addr)
+static uae_u32 REGPARAM2 clock_lget (uaecptr addr)
 {
     return clock_bget(addr+3);
 }
 
-uae_u32 REGPARAM2 clock_wget (uaecptr addr)
+static uae_u32 REGPARAM2 clock_wget (uaecptr addr)
 {
     return clock_bget(addr+1);
 }
 
-uae_u32 REGPARAM2 clock_bget (uaecptr addr)
+static uae_u32 REGPARAM2 clock_bget (uaecptr addr)
 {
-    time_t t=time(0);
+    time_t t = time(NULL);
     struct tm *ct;
     ct=localtime(&t);
     switch (addr & 0x3f)
@@ -748,17 +743,17 @@ uae_u32 REGPARAM2 clock_bget (uaecptr addr)
     return 0;
 }
 
-void REGPARAM2 clock_lput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 clock_lput (uaecptr addr, uae_u32 value)
 {
     /* No way */
 }
 
-void REGPARAM2 clock_wput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 clock_wput (uaecptr addr, uae_u32 value)
 {
     /* No way */
 }
 
-void REGPARAM2 clock_bput (uaecptr addr, uae_u32 value)
+static void REGPARAM2 clock_bput (uaecptr addr, uae_u32 value)
 {
     switch (addr & 0x3f)
     {
