@@ -17,24 +17,21 @@
  Occasional memory reads over buffer ranges can occur, but they will of course
  be fixed when spotted :P The worst that can happen with reading over the
  buffer range is a core dump :)
+
+ TODO: Add ProRunner 1.0 detection. At offset 0x438 string "SNT."
 */
 
+#include <uade/amifilemagic.h>
+#include <uade/ossupport.h>
+#include <uade/uade.h>
+#include <uade/uadeutils.h>
+#include <uade/unixsupport.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdint.h>
 
-#include <uadeutils.h>
-#include <amifilemagic.h>
-
-#define FILEMAGIC_DEBUG 0
-
-#if FILEMAGIC_DEBUG
-#define amifiledebug(fmt, args...) do { fprintf(stderr, "%s:%d: %s: " fmt, __FILE__, __LINE__, __func__, ## args); } while(0)
-#else
-#define amifiledebug(fmt, args...) 
-#endif
-
 #define WAV_HEADER_LEN 44
 
 enum {
@@ -58,8 +55,8 @@ enum {
 #define S31_HEADER_LENGTH 1084
 
 
-static int chk_id_offset(unsigned char *buf, int bufsize,
-			 const char *patterns[], int offset, char *pre);
+static int chk_id_offset(unsigned char *buf, size_t bufsize,
+			 const char *patterns[], size_t offset, char *pre);
 
 
 /* Do not use '\0'. They won't work in patterns */
@@ -117,6 +114,20 @@ static const char *offset_0000_patterns[] = {
   NULL, NULL
 };
 
+struct binary_pattern {
+	size_t off;
+	size_t len;
+	const char *pattern;
+	int not_bad_value;
+	unsigned char bad_value;
+};
+
+static const struct binary_pattern aprosys_pattern[] = {
+	{.off = 0, .len = 11, .pattern = "ADRVPACK\x00\x00\x00"},
+	{.off = 11, .len = 1, .not_bad_value = 1, .bad_value = 0},
+	{.len = 0},
+};
+
 static const char *offset_0024_patterns[] = {
   /* ID: Prefix: Desc: */
   "UNCLEART", "DL",		/* Dave Lowe WT */
@@ -133,19 +144,146 @@ static const char *offset_0024_patterns[] = {
   NULL, NULL
 };
 
+// TODO: Complete list from amifilemagic.
+//       offset_0000_patterns and offset_0024_patterns have been input.
+static struct uade_ext_to_format_version etf[] = {
+	{.file_ext = "abk", .format = "Amos ABK"},
+	{.file_ext = "ahx", .format = "AHX"},
+	{.file_ext = "amc", .format = "A.M.Composer"},
+	{.file_ext = "aon4", .format = "Art Of Noise (4 ch)"},
+	{.file_ext = "aon8", .format = "Art Of Noise (8 ch)"},
+	{.file_ext = "aps", .format = "AProSys"},
+	{.file_ext = "bds", .format = "Benn Daglish"},
+	{.file_ext = "bsi", .format = "Future Composer (BSI)"},
+	{.file_ext = "bss", .format = "Beathoven Synthesizer"},
+	{.file_ext = "core", .format = "Core Design"},
+	{.file_ext = "cust", .format = "Custom"},
+	{.file_ext = "dl", .format = "Dave Lowe"},
+	{.file_ext = "dl_deli", .format = "Dave Lowe"},
+	{.file_ext = "dln", .format = "Dave Lowe New"},
+	{.file_ext = "dm1", .format = "Delta Music", .version = "1"},
+	{.file_ext = "dm2", .format = "Delta Music", .version = "2"},
+	{.file_ext = "digi", .format = "DigiBooster"},
+	{.file_ext = "dss", .format = "DSS"},
+	{.file_ext = "dw", .format = "David Whittaker"},
+	// Note: ems is not recognized yet
+	// {.file_ext = "ems", .format = "Editeur Musical Sequentiel"},
+	{.file_ext = "emsv6", .format = "Editeur Musical Sequentiel",
+	 .version = "6"},
+	{.file_ext = "fc13", .format = "Future Composer", .version = "1.3"},
+	{.file_ext = "fc14", .format = "Future Composer", .version = "1.4"},
+	{.file_ext = "fc-m", .format = "FC-M Packer"},
+	{.file_ext = "fp", .format = "Future Player"},
+	{.file_ext = "ftm", .format = "Face The Music"},
+	{.file_ext = "fuzz", .format = "Fuzzac Packer"},
+	{.file_ext = "glue", .format = "GlueMon"},
+	{.file_ext = "gray", .format = "Fred Gray"},
+	{.file_ext = "hd", .format = "Howie Davies"},
+	{.file_ext = "is", .format = "In Stereo", .version = "1"},
+	{.file_ext = "is20", .format = "In Stereo", .version = "2"},
+	{.file_ext = "jam", .format = "JamCracker"},
+	{.file_ext = "jmf", .format = "Janko Mrsic-Flogel"},
+	{.file_ext = "mxtx", .format = "Maxtrax"},
+	{.file_ext = "mcmd_org", .format = "MCMD"},
+	{.file_ext = "mmdc", .format = "MED Packer"},
+	{.file_ext = "ml", .format = "Musicline Editor"},
+
+	// Pro/Sound/NoiseTracker and variants
+	{.file_ext = "mod_adsc4", .format = "Audio Sculpture (4 ch)"},
+	{.file_ext = "mod_adsc8", .format = "Audio Sculpture (8 ch)"},
+	{.file_ext = "mod_ntk", .format = "NoiseTracker", .version = "1.x"},
+	{.file_ext = "mod_ntkamp", .format = "NoiseTracker"},
+	{.file_ext = "mod_ntk1", .format = "NoiseTracker", .version = "1.2"},
+	{.file_ext = "mod_ntk2", .format = "NoiseTracker", .version = "2.0"},
+	{.file_ext = "mod", .format = "ProTracker"},
+	{.file_ext = "mod_flt4", .format = "StarTrekker (4 ch)"},
+	{.file_ext = "mod_flt8", .format = "StarTrekker (8 ch)"},
+	{.file_ext = "mod_doc", .format = "SoundTracker", .version = "2.4"},
+
+	{.file_ext = "mso", .format = "Medley"},
+	// Note: Check spelling of MultiTracker, find modules
+	{.file_ext = "mtm", .format = "MultiTracker"},
+	{.file_ext = "mtp2", .format = "Marjo Tom's Player"},  // Major Tom
+	{.file_ext = "ntp", .format = "NovoTrade Packer"},
+	{.file_ext = "okt", .format = "Oktalyzer"},
+	{.file_ext = "osp", .format = "Synth Pack"},
+	{.file_ext = "p40a", .format = "The Player", .version = "4.0a"},
+	{.file_ext = "p40b", .format = "The Player", .version = "4.0b"},
+	{.file_ext = "p41a", .format = "The Player", .version = "4.1a"},
+	{.file_ext = "p50a", .format = "The Player", .version = "5.0a"},
+	{.file_ext = "p60a", .format = "The Player", .version = "6.0a"},
+	{.file_ext = "p61a", .format = "The Player", .version = "6.1a"},
+	// Note: pm1 is not recognized yet
+	// {.file_ext = "pm1", .format = "Promizer"},
+	{.file_ext = "pm20", .format = "Promizer", .version = "2.0"},
+	{.file_ext = "pm40", .format = "Promizer", .version = "4.0"},
+	// Note: pru1 is not recognized yet
+	{.file_ext = "pru1", .format = "ProRunner", .version = "1.0"},
+	{.file_ext = "pru2", .format = "ProRunner", .version = "2.0"},
+	{.file_ext = "riff", .format = "Riff Raff"},
+	{.file_ext = "sa", .format = "Sonic Arranger"},
+	{.file_ext = "spl", .format = "Sound Programming Language"},
+	{.file_ext = "stp3", .format = "Soundtracker pro II"},
+	{.file_ext = "synmod", .format = "Syntracker"},
+	{.file_ext = "tp1", .format = "Tracker Packer", .version = "1"},
+	{.file_ext = "tp2", .format = "Tracker Packer", .version = "2"},
+	{.file_ext = "tp3", .format = "Tracker Packer", .version = "3"},
+	{.file_ext = "tpu", .format = "Dirk Bialluch"},
+	{.file_ext = NULL},
+};
+
+
+const struct uade_ext_to_format_version *uade_file_ext_to_format_version(
+	const struct uade_detection_info *info)
+{
+	size_t i;
+	if (info == NULL)
+		return NULL;
+	const char *file_ext = info->ext;
+	for (i = 0;; i++) {
+		if (etf[i].file_ext == NULL)
+			break;
+		if (strcasecmp(etf[i].file_ext, file_ext) == 0)
+			return &etf[i];
+	}
+	return NULL;
+}
 
 /* check for 'pattern' in 'buf'.
    the 'pattern' must lie inside range [0, maxlen) in the buffer.
    returns true if pattern is at buf[offset], otherwrise false
  */
 static int patterntest(const unsigned char *buf, const char *pattern,
-		       int offset, int bytes, int maxlen)
+		       size_t offset, size_t bytes, size_t maxlen)
 {
   if ((offset + bytes) <= maxlen)
-    return (memcmp(buf + offset, pattern, bytes) == 0) ? 1 : 0;
+	  return memcmp(buf + offset, pattern, bytes) == 0;
+
   return 0;
 }
 
+static int check_binary_pattern(unsigned char *buf, size_t bufsize,
+				const struct binary_pattern patterns[])
+{
+	size_t i;
+	for (i = 0; patterns[i].len != 0; i++) {
+		if (patterns[i].not_bad_value) {
+			size_t max_off = patterns[i].off + patterns[i].len;
+			size_t j;
+			if (max_off > bufsize)
+				return 0;
+			for (j = patterns[i].off; j < max_off; j++) {
+				if (buf[j] == patterns[i].bad_value)
+					return 0;
+			}
+		} else if (!patterntest(buf, patterns[i].pattern,
+					patterns[i].off, patterns[i].len,
+					bufsize)) {
+			return 0;
+		}
+	}
+	return 1;
+}
 
 static int tronictest(unsigned char *buf, size_t bufsize)
 {
@@ -162,8 +300,6 @@ static int tronictest(unsigned char *buf, size_t bufsize)
   if (read_be_u32(&buf[a + 4]) != 0x5800b0)
     return 0;
 
-  amifiledebug("tronic recognized\n");
-
   return 1;
 }
 
@@ -304,7 +440,8 @@ static size_t modlentest(unsigned char *buf, size_t bufsize, size_t filesize,
 }
 
 
-static void modparsing(unsigned char *buf, size_t bufsize, size_t header, int max_pattern, int pfx[], int pfxarg[])
+static void modparsing(unsigned char *buf, size_t bufsize, size_t header,
+		       int max_pattern, int pfx[], int pfxarg[])
 {
   int offset;
   int i, j, fx;
@@ -397,7 +534,8 @@ static int mod32check(unsigned char *buf, size_t bufsize, size_t realfilesize,
     return MOD_NOISETRACKER20;		/* Noisetracker2.x */
 
   for (i = 0; startrekker_patterns[i]; i++) {
-    if (patterntest(buf, startrekker_patterns[i], (S31_HEADER_LENGTH - 4), 4, bufsize)) {
+    if (patterntest(buf, startrekker_patterns[i], (S31_HEADER_LENGTH - 4), 4,
+		    bufsize)) {
       t = 0;
       for (j = 0; j < 30 * 0x1e; j = j + 0x1e) {
 	if (buf[0x2a + j] == 0 && buf[0x2b + j] == 0 && buf[0x2d + j] != 0) {
@@ -437,8 +575,7 @@ static int mod32check(unsigned char *buf, size_t bufsize, size_t realfilesize,
     }
 
     if (calculated_size > realfilesize) {
-        fprintf(stderr, "uade: file is truncated and won't get played (%s)\n", path);
-      return MOD_UNDEFINED;
+        fprintf(stderr, "uade: file seems truncated and may get played badly (%s)\n", path);
     }
 
     if (calculated_size < realfilesize) {
@@ -513,9 +650,9 @@ static int mod32check(unsigned char *buf, size_t bufsize, size_t realfilesize,
       /* PC tracker:			0,1,2,3,4,5,6,7,8,9,a,b,c,d,e,f +e## */
 
       for (j = 17; j <= 31; j++) {
-	if (pfx[j] != 0 || finetune_used >0) /* Extended fx used */ {
+	if (pfx[j] != 0 || finetune_used > 0) /* Extended fx used */ {
 	  if (buf[0x3b7] != 0x7f && buf[0x3b7] != 0x78) {
-	    return MOD_FASTTRACKER; /* Definetely Fasttracker*/
+	    return MOD_FASTTRACKER; /* Definitely FastTracker */
 	  } else {
 	    return MOD_PROTRACKER; /* Protracker*/
 	  }
@@ -1070,7 +1207,10 @@ void uade_filemagic(unsigned char *buf, size_t bufsize, char *pre,
     /* generic ID Check at offset 0x00 */
   } else if (chk_id_offset(buf, bufsize, offset_0000_patterns, 0x00, pre)) {
 
-    /*magic ids of some modpackers */
+  } else if (check_binary_pattern(buf, bufsize, aprosys_pattern)) {
+	  strcpy(pre, "APS");  /* AProSys */
+
+    /* magic ids of some modpackers */
   } else if (buf[0x438] == 'P' && buf[0x439] == 'W' && buf[0x43a] == 'R'
 	     && buf[0x43b] == 0x2e) {
     strcpy(pre, "PPK");		/*Polkapacker */
@@ -1089,7 +1229,7 @@ void uade_filemagic(unsigned char *buf, size_t bufsize, char *pre,
 	     && buf[0x3bb] == 'S') {
     strcpy(pre, "KRIS");	/*Kristracker */
 
-  } else if (buf[0] == 'X' && buf[1] == 'P' && buf[2] == 'K' && buf[3] == 'F'&&
+  } else if (buf[0] == 'X' && buf[1] == 'P' && buf[2] == 'K' && buf[3] == 'F' &&
 	     read_be_u32(&buf[4]) + 8 == realfilesize &&
 	     buf[8] == 'S' && buf[9] == 'Q' && buf[10] == 'S' && buf[11] == 'H') {
     fprintf(stderr, "uade: The file is SQSH packed. Please depack first.\n");
@@ -1106,7 +1246,8 @@ void uade_filemagic(unsigned char *buf, size_t bufsize, char *pre,
     /* Custom file check */
   } else if (buf[0] == 0x00 && buf[1] == 0x00 && buf[2] == 0x03
 	     && buf[3] == 0xf3) {
-     /*CUSTOM*/ i = (buf[0x0b] * 4) + 0x1c;	/* beginning of first chunk */
+	  /* CUSTOM */
+	  i = (buf[0x0b] * 4) + 0x1c;	/* beginning of first chunk */
 
     if (i < bufsize - 0x42) {
 
@@ -1148,21 +1289,24 @@ void uade_filemagic(unsigned char *buf, size_t bufsize, char *pre,
 }
 
 
-/* We are currently stupid and check only for a few magic IDs at the offsets
+/*
+ * We are currently stupid and check only for a few magic IDs at the offsets
  * chk_id_offset returns 1 on success and sets the right prefix/extension
  * in pre
+ *
  * TODO: more and less easy check for the rest of the 52 trackerclones
  */
-static int chk_id_offset(unsigned char *buf, int bufsize,
-			 const char *patterns[], int offset, char *pre)
+static int chk_id_offset(unsigned char *buf, size_t bufsize,
+			 const char *patterns[], size_t offset, char *pre)
 {
-  int i;
-  for (i = 0; patterns[i]; i = i + 2) {
-    if (patterntest(buf, patterns[i], offset, strlen(patterns[i]), bufsize)) {
-      /* match found */
-      strcpy(pre, patterns[i + 1]);
-      return 1;
-    }
-  }
-  return 0;
+	int i;
+	for (i = 0; patterns[i]; i = i + 2) {
+		if (patterntest(buf, patterns[i], offset, strlen(patterns[i]),
+				bufsize)) {
+			/* match found */
+			strcpy(pre, patterns[i + 1]);
+			return 1;
+		}
+	}
+	return 0;
 }
