@@ -1,12 +1,24 @@
-/* Handle uade.conf file
-
-   Copyright (C) 2005 Heikki Orsila <heikki.orsila@iki.fi>
-
-   This source code module is dual licensed under GPL and Public Domain.
-   Hence you may use _this_ module (not another code module) in any way you
-   want in your projects.
+/*
+ * Handle uade.conf file
+ *
+ * Copyright (C) 2005 Heikki Orsila <heikki.orsila@iki.fi>
+ *
+ * This source code module is dual licensed under GPL and Public Domain.
+ * Hence you may use _this_ module (not another code module) in any way you
+ * want in your projects.
 */
 
+#include <uade/uade.h>
+#include <uade/uadeconf.h>
+#include <uade/uadeconfstructure.h>
+#include <uade/options.h>
+#include <uade/ossupport.h>
+#include <uade/uadeconstants.h>
+#include <uade/amigafilter.h>
+#include <uade/effects.h>
+#include <uade/uadestate.h>
+#include <support.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -18,15 +30,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "ossupport.h"
-#include "uadeconf.h"
-#include "uadeconfig.h"
-#include "amigafilter.h"
-#include "uadeconstants.h"
-#include "songdb.h"
-#include "uadeutils.h"
-#include "support.h"
-
 static int uade_set_silence_timeout(struct uade_config *uc, const char *value);
 static int uade_set_subsong_timeout(struct uade_config *uc, const char *value);
 static int uade_set_timeout(struct uade_config *uc, const char *value);
@@ -41,10 +44,7 @@ struct uade_conf_opts {
 /* List of uade.conf options. The list includes option name, minimum
    string match length for the option name and its enum code. */
 static const struct uade_conf_opts uadeconfopts[] = {
-	{.str = "action_keys",           .l = 2,  .e = UC_ACTION_KEYS},
 	{.str = "ao_option",             .l = 2,  .e = UC_AO_OPTION},
-	{.str = "buffer_time",           .l = 1,  .e = UC_BUFFER_TIME},
-	{.str = "cygwin",                .l = 1,  .e = UC_CYGWIN_DRIVE_WORKAROUND},
 	{.str = "detect_format_by_detection", .l = 18, .e = UC_CONTENT_DETECTION},
 	{.str = "disable_timeout",       .l = 1,  .e = UC_DISABLE_TIMEOUTS},
 	{.str = "enable_timeout",        .l = 2,  .e = UC_ENABLE_TIMEOUTS},
@@ -64,16 +64,12 @@ static const struct uade_conf_opts uadeconfopts[] = {
 	{.str = "no_ep_end_detect",      .l = 4,  .e = UC_NO_EP_END},
 	{.str = "no_filter",             .l = 4,  .e = UC_NO_FILTER},
 	{.str = "no_song_end",           .l = 4,  .e = UC_NO_EP_END},
-	{.str = "normalise",             .l = 1,  .e = UC_NORMALISE},
 	{.str = "ntsc",                  .l = 2,  .e = UC_NTSC},
 	{.str = "one_subsong",           .l = 1,  .e = UC_ONE_SUBSONG},
 	{.str = "pal",                   .l = 3,  .e = UC_PAL},
 	{.str = "panning_value",         .l = 3,  .e = UC_PANNING_VALUE},
-	{.str = "random_play",           .l = 3,  .e = UC_RANDOM_PLAY},
-	{.str = "recursive_mode",        .l = 3,  .e = UC_RECURSIVE_MODE},
-	{.str = "resampler",             .l = 3,  .e = UC_RESAMPLER},
+	{.str = "resampler",             .l = 1,  .e = UC_RESAMPLER},
 	{.str = "silence_timeout_value", .l = 2,  .e = UC_SILENCE_TIMEOUT_VALUE},
-	{.str = "song_title",            .l = 2,  .e = UC_SONG_TITLE},
 	{.str = "speed_hack",            .l = 2,  .e = UC_SPEED_HACK},
 	{.str = "subsong_timeout_value", .l = 2,  .e = UC_SUBSONG_TIMEOUT_VALUE},
 	{.str = "timeout_value",         .l = 1,  .e = UC_TIMEOUT_VALUE},
@@ -95,13 +91,40 @@ static enum uade_option map_str_to_option(const char *key)
 	return 0;
 }
 
+struct uade_config *uade_new_config(void)
+{
+	struct uade_config *uc = calloc(1, sizeof *uc);
+	if (uc)
+		uade_config_set_defaults(uc);
+	return uc;
+}
+
+static void uade_set_filter_type(struct uade_config *uc, const char *model)
+{
+	uc->filter_type = FILTER_MODEL_A500;
+
+	if (model == NULL)
+		return;
+
+	/* a500 and a500e are the same */
+	if (strncasecmp(model, "a500", 4) == 0) {
+		uc->filter_type = FILTER_MODEL_A500;
+
+		/* a1200 and a1200e are the same */
+	} else if (strncasecmp(model, "a1200", 5) == 0) {
+		uc->filter_type = FILTER_MODEL_A1200;
+
+	} else {
+		fprintf(stderr, "Unknown filter model: %s\n", model);
+	}
+}
+
 /* The function sets the default options. No *_set variables are set because
    we don't want any option to become mergeable by default. See
    uade_merge_configs(). */
 void uade_config_set_defaults(struct uade_config *uc)
 {
 	memset(uc, 0, sizeof(*uc));
-	uc->action_keys = 1;
 	strlcpy(uc->basedir.name, UADE_CONFIG_BASE_DIR,	sizeof uc->basedir.name);
 	uade_set_filter_type(uc, NULL);
 	uc->frequency = UADE_DEFAULT_FREQUENCY;
@@ -116,7 +139,8 @@ void uade_config_set_defaults(struct uade_config *uc)
 double uade_convert_to_double(const char *value, double def, double low,
 			      double high, const char *type)
 {
-	char *endptr, *newvalue;
+	char *convertedvalue = NULL;
+	char *endptr;
 	char newseparator;
 	double v;
 
@@ -127,23 +151,26 @@ double uade_convert_to_double(const char *value, double def, double low,
 
 	/* Decimal separator conversion, if needed */
 	if (*endptr == ',' || *endptr == '.') {
-		newvalue = strdup(value);
-		if (newvalue == NULL)
-			uade_error("Out of memory\n");
+		convertedvalue = strdup(value);
+		if (convertedvalue == NULL) {
+			uade_warning("Out of memory\n");
+			return def;
+		}
 
 		newseparator = (*endptr == ',') ? '.' : ',';
 
-		newvalue[(intptr_t) endptr - (intptr_t) value] = newseparator;
+		convertedvalue[(intptr_t) endptr - (intptr_t) value] = newseparator;
 
-		v = strtod(newvalue, &endptr);
-		free(newvalue);
+		v = strtod(convertedvalue, &endptr);
 	}
 
 	if (*endptr != 0 || v < low || v > high) {
-		fprintf(stderr, "Invalid %s value: %s\n", type, value);
+		uade_warning("Invalid %s value: %s\n", type, value);
 		v = def;
 	}
 
+	free_and_null(convertedvalue);
+
 	return v;
 }
 
@@ -159,63 +186,60 @@ static void uade_add_ep_option(struct uade_ep_options *opts, const char *s)
 	opts->s += strlen(s) + 1;
 }
 
-static int handle_attributes(struct uade_config *uc, struct uade_song *us,
-			     char *playername, size_t playernamelen,
-			     int flags, struct uade_attribute *attributelist)
+static void uade_add_ao_option(struct uade_ao_options *opts, const char *s)
 {
-	struct uade_attribute *a;
-	size_t i;
-
-	for (i = 0; epconf[i].s != NULL; i++) {
-
-		if (epconf[i].o == 0)
-			continue;
-
-		if ((flags & epconf[i].e) == 0)
-			continue;
-
-		uade_set_config_option(uc, epconf[i].o, epconf[i].c);
+	const size_t max_size = sizeof(opts->o);
+	if (strlen(opts->o) > 0) {
+		if (strlcat(opts->o, ",", max_size) >= max_size) {
+			fprintf(stderr,
+				"Warning: uade ao option overflow: %s\n", s);
+			return;
+		}
 	}
+	if (strlcat(opts->o, s, max_size) >= max_size) {
+		fprintf(stderr, "Warning: uade ao option overflow: %s\n", s);
+		return;
+	}
+}
 
-	if (flags & ES_NEVER_ENDS)
-		fprintf(stderr, "uade: ES_NEVER_ENDS is not implemented. What should it do?\n");
+static int set_options_from_attributes(struct uade_state *state,
+				       char *playername,
+				       size_t playernamelen,
+				       struct uade_attribute *attributes)
+{
+	struct uade_song_state *us = &state->song;
+	struct uade_config *uc = &state->config;
 
-	if (flags & ES_REJECT)
-		return -1;
-
-	a = attributelist;
-
-	while (a != NULL) {
-
-		switch (a->type) {
+	for (; attributes != NULL; attributes = attributes->next) {
+		switch (attributes->flag) {
 		case ES_EP_OPTION:
-			if (uc->verbose)
-				fprintf(stderr, "Using eagleplayer option %s\n", a->s);
-			uade_add_ep_option(&us->ep_options, a->s);
+			uade_debug(state, "Using eagleplayer option %s\n",
+				   attributes->s);
+			uade_add_ep_option(&us->ep_options, attributes->s);
 			break;
 
 		case ES_GAIN:
-			uade_set_config_option(uc, UC_GAIN, a->s);
+			uade_config_set_option(uc, UC_GAIN, attributes->s);
 			break;
 
 		case ES_RESAMPLER:
-			uade_set_config_option(uc, UC_RESAMPLER, a->s);
+			uade_config_set_option(uc, UC_RESAMPLER, attributes->s);
 			break;
 
 		case ES_PANNING:
-			uade_set_config_option(uc, UC_PANNING_VALUE, a->s);
+			uade_config_set_option(uc, UC_PANNING_VALUE, attributes->s);
 			break;
 
 		case ES_PLAYER:
 			if (playername) {
-				snprintf(playername, playernamelen, "%s/players/%s", uc->basedir.name, a->s);
+				snprintf(playername, playernamelen, "%s/players/%s", uc->basedir.name, attributes->s);
 			} else {
 				fprintf(stderr, "Error: attribute handling was given playername == NULL.\n");
 			}
 			break;
 
 		case ES_SILENCE_TIMEOUT:
-			uade_set_config_option(uc, UC_SILENCE_TIMEOUT_VALUE, a->s);
+			uade_config_set_option(uc, UC_SILENCE_TIMEOUT_VALUE, attributes->s);
 			break;
 
 		case ES_SUBSONGS:
@@ -223,126 +247,125 @@ static int handle_attributes(struct uade_config *uc, struct uade_song *us,
 			break;
 
 		case ES_SUBSONG_TIMEOUT:
-			uade_set_config_option(uc, UC_SUBSONG_TIMEOUT_VALUE, a->s);
+			uade_config_set_option(uc, UC_SUBSONG_TIMEOUT_VALUE, attributes->s);
 			break;
 
 		case ES_TIMEOUT:
-			uade_set_config_option(uc, UC_TIMEOUT_VALUE, a->s);
+			uade_config_set_option(uc, UC_TIMEOUT_VALUE, attributes->s);
 			break;
 
 		default:
-			fprintf(stderr,	"Unknown song attribute integer: 0x%x\n", a->type);
+			fprintf(stderr,	"Unknown song attribute flag: 0x%x\n", attributes->flag);
 			break;
 		}
-
-		a = a->next;
 	}
 
 	return 0;
 }
 
-int uade_set_song_attributes(struct uade_state *state,
-			     char *playername, size_t playernamelen)
+int uade_set_options_from_song_attributes(struct uade_state *state,
+					  char *playername,
+					  size_t playernamelen)
 {
-	struct uade_song *us = state->song;
-	struct uade_config *uc = &state->config;
+	/* Look at flags and set uade config options accordingly */
+	if (uade_set_config_options_from_flags(state, state->song.flags))
+		uade_warning("uade_set_song_attributes failed when setting config options from flags\n");
 
-	if (us->normalisation)
-		uade_set_config_option(uc, UC_NORMALISE, us->normalisation);
-
-	return handle_attributes(uc, us, playername, playernamelen,
-				 us->flags, us->songattributes);
+	return set_options_from_attributes(state,
+					   playername, playernamelen,
+					   state->song.songattributes);
 }
 
-int uade_load_config(struct uade_config *uc, const char *filename)
+int uade_load_config(struct uade_state *state, const char *filename)
 {
 	char line[256];
 	FILE *f;
 	char *key, *value;
 	int linenumber = 0;
 	enum uade_option opt;
+	struct uade_config *uc = &state->permconfig;
+
+	state->permconfigname[0] = 0;
 
 	if ((f = fopen(filename, "r")) == NULL)
 		return 0;
 
 	uade_config_set_defaults(uc);
 
-	while (xfgets(line, sizeof(line), f) != NULL) {
+	while (uade_xfgets(line, sizeof(line), f) != NULL) {
 		linenumber++;
 
 		/* Skip comment lines */
 		if (line[0] == '#')
 			continue;
 
-		if (!get_two_ws_separated_fields(&key, &value, line))
+		if (!uade_get_two_ws_separated_fields(&key, &value, line))
 			continue; /* Skip an empty line */
 
 		opt = map_str_to_option(key);
 
 		if (opt) {
-			uade_set_config_option(uc, opt, value);
+			uade_config_set_option(uc, opt, value);
 		} else {
-			fprintf(stderr,	"Unknown config key in %s on line %d: %s\n", filename, linenumber, key);
+			fprintf(stderr,
+				"Unknown config key in %s on line %d: %s\n",
+				filename, linenumber, key);
 		}
 	}
 
 	fclose(f);
+
+	snprintf(state->permconfigname, sizeof(state->permconfigname), "%s",
+		 filename);
+
 	return 1;
 }
 
-int uade_load_initial_config(char *uadeconfname, size_t maxlen,
-			     struct uade_config *uc, struct uade_config *ucbase)
+int uade_load_initial_config(struct uade_state *state, const char *bdir)
 {
-	int loaded;
+	int loaded = 0;
 	char *home;
+	char tmpname[PATH_MAX];
 
-	assert(maxlen > 0);
-	uadeconfname[0] = 0;
-
-	uade_config_set_defaults(uc);
-
-	loaded = 0;
+	state->permconfigname[0] = 0;
+	uade_config_set_defaults(&state->permconfig);
 
 	/* First try to load from forced base dir (testing mode) */
-	if (ucbase != NULL && ucbase->basedir_set) {
-		snprintf(uadeconfname, maxlen, "%s/uade.conf",
-			 ucbase->basedir.name);
-		loaded = uade_load_config(uc, uadeconfname);
+	if (bdir != NULL) {
+		snprintf(tmpname, sizeof(tmpname), "%s/uade.conf", bdir);
+		loaded = uade_load_config(state, tmpname);
 	}
 
+	/* Second, try to load config from ~/.uade/uade.conf */
 	home = uade_open_create_home();
-
-	/* Second, try to load config from ~/.uade2/uade.conf */
 	if (loaded == 0 && home != NULL) {
-		snprintf(uadeconfname, maxlen, "%s/.uade2/uade.conf", home);
-		loaded = uade_load_config(uc, uadeconfname);
+		snprintf(tmpname, sizeof(tmpname), "%s/.uade/uade.conf", home);
+		loaded = uade_load_config(state, tmpname);
 	}
 
 	/* Third, try to load from install path */
 	if (loaded == 0) {
-		snprintf(uadeconfname, maxlen, "%s/uade.conf",
-			 uc->basedir.name);
-		loaded = uade_load_config(uc, uadeconfname);
+		snprintf(tmpname, sizeof(tmpname), "%s/uade.conf", state->permconfig.basedir.name);
+		loaded = uade_load_config(state, tmpname);
 	}
 
+	state->config = state->permconfig;
+
 	return loaded;
 }
 
-int uade_load_initial_song_conf(char *songconfname, size_t maxlen,
-				struct uade_config *uc,
-				struct uade_config *ucbase)
+int uade_load_initial_song_conf(struct uade_state *state)
 {
 	int loaded = 0;
 	char *home;
-
-	assert(maxlen > 0);
-	songconfname[0] = 0;
+	char tmpname[PATH_MAX];
+	struct uade_config *uc = &state->config;
 
 	/* Used for testing */
-	if (ucbase != NULL && ucbase->basedir_set) {
-		snprintf(songconfname, maxlen, "%s/song.conf",
-			 ucbase->basedir.name);
-		loaded = uade_read_song_conf(songconfname);
+	if (uc != NULL && uc->basedir_set) {
+		snprintf(tmpname, sizeof tmpname, "%s/song.conf",
+			 uc->basedir.name);
+		loaded = uade_read_song_conf(tmpname, state);
 	}
 
 	/* Avoid unwanted home directory creation for test mode */
@@ -353,15 +376,14 @@ int uade_load_initial_song_conf(char *songconfname, size_t maxlen,
 
 	/* Try to load from home dir */
 	if (loaded == 0 && home != NULL) {
-		snprintf(songconfname, maxlen, "%s/.uade2/song.conf", home);
-		loaded = uade_read_song_conf(songconfname);
+		snprintf(tmpname, sizeof(tmpname), "%s/.uade/song.conf", home);
+		loaded = uade_read_song_conf(tmpname, state);
 	}
 
-	/* No? Try install path */
+	/* No? Try install path.. */
 	if (loaded == 0) {
-		snprintf(songconfname, maxlen, "%s/song.conf",
-			 uc->basedir.name);
-		loaded = uade_read_song_conf(songconfname);
+		snprintf(tmpname, sizeof(tmpname), "%s/song.conf", state->permconfig.basedir.name);
+		loaded = uade_read_song_conf(tmpname, state);
 	}
 
 	return loaded;
@@ -371,12 +393,9 @@ void uade_merge_configs(struct uade_config *ucd, const struct uade_config *ucs)
 {
 #define MERGE_OPTION(y) do { if (ucs->y##_set) ucd->y = ucs->y; } while (0)
 
-	MERGE_OPTION(action_keys);
 	MERGE_OPTION(ao_options);
 	MERGE_OPTION(basedir);
-	MERGE_OPTION(buffer_time);
 	MERGE_OPTION(content_detection);
-	MERGE_OPTION(cygwin_drive_workaround);
 	MERGE_OPTION(ep_options);
 	MERGE_OPTION(filter_type);
 	MERGE_OPTION(frequency);
@@ -391,25 +410,19 @@ void uade_merge_configs(struct uade_config *ucd, const struct uade_config *ucs)
 	MERGE_OPTION(no_filter);
 	MERGE_OPTION(no_postprocessing);
 
-	/* Special merge -> don't use MERGE_OPTION macro */
-	if (ucs->normalise_set && ucs->normalise) {
-		ucd->normalise = 1;
-		if (ucs->normalise_parameter != NULL)
-			ucd->normalise_parameter = ucs->normalise_parameter;
-	}
-
 	MERGE_OPTION(one_subsong);
 	MERGE_OPTION(panning);
 	MERGE_OPTION(panning_enable);
-	MERGE_OPTION(random_play);
-	MERGE_OPTION(recursive_mode);
+	MERGE_OPTION(player_file);
 	MERGE_OPTION(resampler);
+	MERGE_OPTION(score_file);
 	MERGE_OPTION(silence_timeout);
-	MERGE_OPTION(song_title);
 	MERGE_OPTION(speed_hack);
 	MERGE_OPTION(subsong_timeout);
 
 	MERGE_OPTION(timeout);
+	MERGE_OPTION(uadecore_file);
+	MERGE_OPTION(uae_config_file);
 	MERGE_OPTION(use_timeouts);
 	if (ucs->timeout_set) {
 		ucd->use_timeouts = 1;
@@ -419,16 +432,18 @@ void uade_merge_configs(struct uade_config *ucd, const struct uade_config *ucs)
 	MERGE_OPTION(use_text_scope);
 	MERGE_OPTION(use_ntsc);
 	MERGE_OPTION(verbose);
+
+	MERGE_OPTION(write_audio_file);
 }
 
 char *uade_open_create_home(void)
 {
-	/* Create ~/.uade2 directory if it does not exist */
+	/* Create ~/.uade directory if it does not exist */
 	char *home = getenv("HOME");
 	if (home) {
 		char name[PATH_MAX];
 		struct stat st;
-		snprintf(name, sizeof name, "%s/.uade2", home);
+		snprintf(name, sizeof name, "%s/.uade", home);
 		if (stat(name, &st) != 0)
 			mkdir(name, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
@@ -483,39 +498,40 @@ int uade_parse_subsongs(int **subsongs, char *option)
 
 void uade_set_effects(struct uade_state *state)
 {
-	struct uade_effect *effects = &state->effects;
 	struct uade_config *uc = &state->config;
 
-	uade_effect_set_defaults(effects);
+	uade_effect_set_defaults(state);
 
 	if (uc->no_postprocessing)
-		uade_effect_disable(effects, UADE_EFFECT_ALLOW);
+		uade_effect_disable(state, UADE_EFFECT_ALLOW);
 
 	if (uc->gain_enable) {
-		uade_effect_gain_set_amount(effects, uc->gain);
-		uade_effect_enable(effects, UADE_EFFECT_GAIN);
+		uade_effect_gain_set_amount(state, uc->gain);
+		uade_effect_enable(state, UADE_EFFECT_GAIN);
 	}
 
 	if (uc->headphones)
-		uade_effect_enable(effects, UADE_EFFECT_HEADPHONES);
+		uade_effect_enable(state, UADE_EFFECT_HEADPHONES);
 
 	if (uc->headphones2)
-		uade_effect_enable(effects, UADE_EFFECT_HEADPHONES2);
-
-	if (uc->normalise) {
-		uade_effect_normalise_unserialise(uc->normalise_parameter);
-		uade_effect_enable(effects, UADE_EFFECT_NORMALISE);
-	}
+		uade_effect_enable(state, UADE_EFFECT_HEADPHONES2);
 
 	if (uc->panning_enable) {
-		uade_effect_pan_set_amount(effects, uc->panning);
-		uade_effect_enable(effects, UADE_EFFECT_PAN);
+		uade_effect_pan_set_amount(state, uc->panning);
+		uade_effect_enable(state, UADE_EFFECT_PAN);
 	}
 
-	uade_effect_set_sample_rate(effects, uc->frequency);
+	uade_effect_set_sample_rate(state, uc->frequency);
 }
 
-void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
+static void handle_config_path(struct uade_path *path, char *set,
+			       const char *value)
+{
+	strlcpy(path->name, value, sizeof path->name);
+	*set = 1;
+}
+
+void uade_config_set_option(struct uade_config *uc, enum uade_option opt,
 			    const char *value)
 {
 	char *endptr;
@@ -524,61 +540,24 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 #define SET_OPTION(opt, value) do { uc->opt = (value); uc->opt##_set = 1; } while (0)
 
 	switch (opt) {
-	case UC_ACTION_KEYS:
-		if (value != NULL) {
-			uc->action_keys_set = 1;
-			if (!strcasecmp(value, "on") || !strcmp(value, "1")) {
-				uc->action_keys = 1;
-			} else if (!strcasecmp(value, "off") ||
-				   !strcmp(value, "0")) {
-				uc->action_keys = 0;
-			} else {
-				fprintf(stderr,
-					"uade.conf: Unknown setting for action keys: %s\n",
-					value);
-			}
-		}
-		break;
-
 	case UC_AO_OPTION:
-		strlcat(uc->ao_options.o, value, sizeof uc->ao_options.o);
-		strlcat(uc->ao_options.o, "\n", sizeof uc->ao_options.o);
-		uc->ao_options_set = 1;
+		if (value != NULL) {
+			uade_add_ao_option(&uc->ao_options, value);
+			uc->ao_options_set = 1;
+		} else {
+			fprintf(stderr,
+				"uade: Passed NULL to UC_AO_OPTION.\n");
+		}
 		break;
 
 	case UC_BASE_DIR:
-		if (value != NULL) {
-			strlcpy(uc->basedir.name, value,
-				sizeof uc->basedir.name);
-			uc->basedir_set = 1;
-		} else {
-			fprintf(stderr, "uade: Passed NULL to UC_BASE_DIR.\n");
-		}
-		break;
-
-	case UC_BUFFER_TIME:
-		if (value != NULL) {
-			uc->buffer_time_set = 1;
-			uc->buffer_time = strtol(value, &endptr, 10);
-			if (uc->buffer_time <= 0 || *endptr != 0) {
-				fprintf(stderr, "Invalid buffer_time: %s\n",
-					value);
-				uc->buffer_time = 0;
-			}
-		} else {
-			fprintf(stderr,
-				"uade: Passed NULL to UC_BUFFER_TIME.\n");
-		}
+		handle_config_path(&uc->basedir, &uc->basedir_set, value);
 		break;
 
 	case UC_CONTENT_DETECTION:
 		SET_OPTION(content_detection, 1);
 		break;
 
-	case UC_CYGWIN_DRIVE_WORKAROUND:
-		SET_OPTION(cygwin_drive_workaround, 1);
-		break;
-
 	case UC_DISABLE_TIMEOUTS:
 		SET_OPTION(use_timeouts, 0);
 		break;
@@ -619,8 +598,8 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		}
 		if (strcasecmp(value, "off") == 0 || strcmp(value, "0") == 0) {
 			uc->led_state = 0;
-		} else if (strcasecmp(value, "on") == 0
-			   || strcmp(value, "1") == 0) {
+		} else if (strcasecmp(value, "on") == 0 ||
+			   strcmp(value, "1") == 0) {
 			uc->led_state = 1;
 		} else {
 			fprintf(stderr, "Unknown force led argument: %s\n",
@@ -628,7 +607,6 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 			break;
 		}
 		uc->led_state_set = 1;
-
 		SET_OPTION(led_forced, 1);
 		break;
 
@@ -716,15 +694,6 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		SET_OPTION(no_postprocessing, 1);
 		break;
 
-	case UC_NORMALISE:
-		if (value == NULL) {
-			fprintf(stderr, "uade: UC_NORMALISE is NULL\n");
-			break;
-		}
-		SET_OPTION(normalise, 1);
-		uc->normalise_parameter = (char *) value;
-		break;
-
 	case UC_NTSC:
 		SET_OPTION(use_ntsc, 1);
 		break;
@@ -746,12 +715,12 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		SET_OPTION(panning, uade_convert_to_double(value, 0.0, 0.0, 2.0, "panning"));
 		break;
 
-	case UC_RANDOM_PLAY:
-		SET_OPTION(random_play, 1);
+	case UC_PLAYER_FILE:
+		handle_config_path(&uc->player_file, &uc->player_file_set, value);
 		break;
 
-	case UC_RECURSIVE_MODE:
-		SET_OPTION(recursive_mode, 1);
+	case UC_SCORE_FILE:
+		handle_config_path(&uc->score_file, &uc->score_file_set, value);
 		break;
 
 	case UC_SILENCE_TIMEOUT_VALUE:
@@ -763,18 +732,6 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		uade_set_silence_timeout(uc, value);
 		break;
 
-	case UC_SONG_TITLE:
-		if (value == NULL) {
-			fprintf(stderr, "uade: No song_title format given.\n");
-			break;
-		}
-		if ((uc->song_title = strdup(value)) == NULL) {
-			fprintf(stderr, "No memory for song title format\n");
-		} else {
-			uc->song_title_set = 1;
-		}
-		break;
-
 	case UC_SPEED_HACK:
 		SET_OPTION(speed_hack, 1);
 		break;
@@ -796,6 +753,16 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		uade_set_timeout(uc, value);
 		break;
 
+	case UC_UADECORE_FILE:
+		handle_config_path(&uc->uadecore_file,
+				   &uc->uadecore_file_set, value);
+		break;
+
+	case UC_UAE_CONFIG_FILE:
+		handle_config_path(&uc->uae_config_file,
+				   &uc->uae_config_file_set, value);
+		break;
+
 	case UC_USE_TEXT_SCOPE:
 		SET_OPTION(use_text_scope, 1);
 		break;
@@ -804,36 +771,41 @@ void uade_set_config_option(struct uade_config *uc, enum uade_option opt,
 		SET_OPTION(verbose, 1);
 		break;
 
+	case UC_WRITE_AUDIO_FILE:
+		handle_config_path(&uc->write_audio_file,
+				   &uc->write_audio_file_set, value);
+		break;
+
 	default:
-		fprintf(stderr, "uade_set_config_option(): unknown enum: %d\n",
+		fprintf(stderr, "uade_config_set_option(): unknown enum: %d\n",
 			opt);
 		exit(1);
 	}
 }
 
-void uade_set_ep_attributes(struct uade_state *state)
+int uade_config_toggle_boolean(struct uade_config *uc, enum uade_option opt)
 {
-	handle_attributes(&state->config, state->song, NULL, 0, state->ep->flags, state->ep->attributelist);
+	if (opt == UC_VERBOSE) {
+		uc->verbose ^= 1;
+		return uc->verbose;
+	} else if (opt == UC_FORCE_LED) {
+		uade_config_set_option(uc, UC_FORCE_LED,
+				       uc->led_state ? "off" : "on");
+		return uc->led_state;
+	}
+	return -1;
 }
 
-void uade_set_filter_type(struct uade_config *uc, const char *model)
+void uade_set_options_from_ep_attributes(struct uade_state *state)
 {
-	uc->filter_type = FILTER_MODEL_A500;
+	struct eagleplayer *ep = state->song.info.detectioninfo.ep;
 
-	if (model == NULL)
-		return;
+	/* Look at flags and set uade config options accordingly */
+	if (uade_set_config_options_from_flags(state, ep->flags))
+		uade_warning("uade_set_ep_attributes failed with setting config options from flags\n");
 
-	/* a500 and a500e are the same */
-	if (strncasecmp(model, "a500", 4) == 0) {
-		uc->filter_type = FILTER_MODEL_A500;
-
-		/* a1200 and a1200e are the same */
-	} else if (strncasecmp(model, "a1200", 5) == 0) {
-		uc->filter_type = FILTER_MODEL_A1200;
-
-	} else {
-		fprintf(stderr, "Unknown filter model: %s\n", model);
-	}
+	if (set_options_from_attributes(state, NULL, 0, ep->attributelist))
+		uade_warning("uade_set_ep_attributes failed with setting config options from eagleplayer attributes\n");
 }
 
 static int uade_set_silence_timeout(struct uade_config *uc, const char *value)
