@@ -1,7 +1,9 @@
 #ifndef _TEXT_SCOPE_H_
 #define _TEXT_SCOPE_H_
 
-#include "uadeconfig.h"
+#include "write_audio.h"
+
+#include <uade/options.h>
 
 #ifdef UADE_CONFIG_TEXT_SCOPE
 #define TEXT_SCOPE(cycles, voice, e, value)      \
@@ -13,9 +15,7 @@
 #define TEXT_SCOPE(cycles, voice, e, value) do {} while (0)
 #endif
 
-enum PaulaEventType {PET_VOL, PET_PER, PET_DAT, PET_LEN, PET_LCH, PET_LCL};
-
-void text_scope(unsigned long cycles, int voice, enum PaulaEventType e,
+void text_scope(unsigned long cycles, int voice, enum UADEPaulaEventType e,
 		int value);
 
 #endif
