@@ -11,7 +11,6 @@
 #include "sysconfig.h"
 #include "sysdeps.h"
 
-#include "config.h"
 #include "options.h"
 #include "events.h"
 #include "uae.h"
@@ -24,7 +23,9 @@
 
 #include "cia.h"
 
-#include "uade.h"
+#include "uadectl.h"
+#include <uade/uadeipc.h>
+
 
 #define EXCEPTION_COUNT 0
 #if EXCEPTION_COUNT
@@ -105,7 +106,7 @@ void dump_counts (void)
 
 int broken_in;
 
-static __inline__ unsigned int cft_map (unsigned int f)
+static inline unsigned int cft_map (unsigned int f)
 {
 #ifndef HAVE_GET_WORD_UNSWAPPED
     return f;
@@ -671,7 +672,7 @@ void Exception(int nr, uaecptr oldpc)
 {
     /* trap #5 indicates sound core is sending message for uade */
     if(nr == (32+5)) {
-      uade_get_amiga_message();
+      uadecore_get_amiga_message();
     }
 #if EXCEPTION_COUNT
     exception_freq[nr]++;
@@ -753,6 +754,9 @@ static void Interrupt(int nr)
     lastint_no = nr;
     Exception(nr+24, 0);
 
+    if (debugging)
+	debug_interrupt_happened = 1;
+
     regs.intmask = nr;
     regs.spcflags |= SPCFLAG_INT;
 }
@@ -799,7 +803,7 @@ void m68k_movec2 (int regno, uae_u32 *regp)
 	}
 }
 
-static __inline__ int
+static inline int
 div_unsigned(uae_u32 src_hi, uae_u32 src_lo, uae_u32 div, uae_u32 *quot, uae_u32 *rem)
 {
 	uae_u32 q = 0, cbit = 0;
@@ -945,7 +949,7 @@ void m68k_divl (uae_u32 opcode, uae_u32 src, uae_u16 extra, uaecptr oldpc)
 #endif
 }
 
-static __inline__ void
+static inline void
 mul_unsigned(uae_u32 src1, uae_u32 src2, uae_u32 *dst_hi, uae_u32 *dst_lo)
 {
 	uae_u32 r0 = (src1 & 0xffff) * (src2 & 0xffff);
@@ -1056,11 +1060,6 @@ static char* ccnames[] =
 
 void m68k_reset (void)
 {
-  m68k_areg(regs,7) = 0x1000;
-  m68k_setpc(0x1000);
-
-  uade_prerun();
-
 #if EXCEPTION_COUNT
   {
     int i;
@@ -1091,7 +1090,7 @@ unsigned long REGPARAM2 op_illg (uae_u32 opcode)
 {
     uaecptr pc = m68k_getpc ();
 
-    if (uade_debug) {
+    if (uadecore_debug) {
       fprintf(stderr, "op_illg(): going into debug\n");
       activate_debugger();
     }
@@ -1201,26 +1200,9 @@ static void do_trace (void)
 
 static int do_specialties (void)
 {
-  // THIS WAS ALREADY COMMENTED (T:SHd)
-    /*n_spcinsns++;*/
-
-  /*
-    while (regs.spcflags & SPCFLAG_BLTNASTY) {
-    do_cycles (4);
-    if (regs.spcflags & SPCFLAG_DISK)
-    do_disk ();
-    }
-  */
-
-    //    run_compiled_code();
-
-    /*
-      if (regs.spcflags & SPCFLAG_DOTRACE) {
-      Exception (9,last_trace_ad);
-      }
-    */
-
     while (regs.spcflags & SPCFLAG_STOP) {
+        if (uadecore_reboot)
+	    return 1;
 	do_cycles (4);
 	if (regs.spcflags & (SPCFLAG_INT | SPCFLAG_DOINT)){
 	    int intr = intlev ();
@@ -1232,12 +1214,6 @@ static int do_specialties (void)
 	    }
 	}
     }
-    /* if (regs.spcflags & SPCFLAG_TRACE)
-	 do_trace ();
-
-       if (regs.spcflags & SPCFLAG_DISK)
-	 do_disk ();
-    */
 
     if (regs.spcflags & SPCFLAG_DOINT) {
 	int intr = intlev ();
@@ -1269,7 +1245,7 @@ void m68k_run_1 (void)
   int uade_insts = 0;
 #endif
 
-  for (;;) {
+  while (1) {
 
     opcode = GET_OPCODE;
 
@@ -1282,7 +1258,7 @@ void m68k_run_1 (void)
     
     cycles = (*cpufunctbl[opcode])(opcode);
 
-    if (uade_time_critical)
+    if (uadecore_time_critical)
       cycles = 1;
 
     /*n_insns++;*/
@@ -1292,64 +1268,64 @@ void m68k_run_1 (void)
 
     if (regs.spcflags) {
       if (do_specialties ())
-	return;
+	break;
     }
 
+    if (uadecore_reboot)
+      break;
+
 #if EXCEPTION_COUNT
     uade_insts++;
     uade_revs++;
-    if(uade_revs >= 1000) {
+    if (uade_revs >= 1000) {
       int i;
       uade_revs -= 1000;
       i = time(0);
-      if((i - uade_otime) > 0) {
+      if ((i - uade_otime) > 0) {
 	uade_otime = i;
 	/* fprintf(stderr, "insts: %d\n", uade_insts); */
 	uade_insts = 0;
 	fprintf(stderr, "exceptions: ");
-	for(i=1; i<64; i++) {
-	  if(exception_freq[i])
+	for (i=1; i<64; i++) {
+	  if (exception_freq[i])
 	    fprintf(stderr, "%.3x: %.8d ", i*4, exception_freq[i]);
 	}
 	fprintf(stderr, "\n");
       }
     }
 #endif
-
-    if(uade_reboot)
-      return;
   }
 }
 
 int in_m68k_go = 0;
 
-void m68k_go (int may_quit)
+void m68k_go (void)
 {
-  if (in_m68k_go || !may_quit) {
-    write_log ("Bug! m68k_go is not reentrant.\n");
-    abort ();
-  }
-
   reset_frame_rate_hack ();
   update_68k_cycles ();
-  
-  in_m68k_go++;
 
-  for (;quit_program != 1;) {
-    quit_program = 0;
+  while (quit_program == 0) {
+    uadecore_reset ();
     m68k_reset ();
     customreset ();
 
-    while(uade_reboot == 0 && quit_program != 1) {
-      if (debugging) {
+    uadecore_handle_r_state();
+
+    while (uadecore_reboot == 0 && quit_program == 0) {
+      if (debugging)
 	debug ();
-	if (quit_program == 1)
-	  break;
-      }
+      if (quit_program != 0)
+	break;
       m68k_run_1 ();
     }
+
+    if (uadecore_reboot) {
+      if (uade_send_short_message(UADE_COMMAND_TOKEN, &uadecore_ipc) < 0) {
+	fprintf(stderr, "can not send reboot ack token\n");
+	exit(1);
+      }
+    }
   }
-  in_m68k_go--;
 }
 
 static void m68k_verify (uaecptr addr, uaecptr *nextpc)
