@@ -8,7 +8,8 @@
 
 #include "text_scope.h"
 
-static char *PaulaEventStr[] = {"Vol", "Per", "Dat", "Len", "LCL", "LCH"};
+static char *PaulaEventStr[] = {
+	"None", "Vol", "Per", "Dat", "Len", "LCL", "LCH"};
 
 
 static void print_tab(int v)
@@ -23,11 +24,12 @@ static void print_tab(int v)
 }
 
 
-void text_scope(unsigned long cycles, int voice, enum PaulaEventType e,
+void text_scope(unsigned long cycles, int voice, enum UADEPaulaEventType e,
 		int value)
 {
     unsigned int sec = cycles / SOUNDTICKS;
-    unsigned int usec = ((uint64_t) (cycles % SOUNDTICKS) * 1000000) / SOUNDTICKS;
+    unsigned int usec = (((uint64_t) (cycles % SOUNDTICKS) * 1000000) /
+			 SOUNDTICKS);
 
     printf("%6u:%.6u ", sec & 0x1ffff, usec);
     print_tab(voice);
